@@ -23,6 +23,7 @@ Module to load data into typed data structures
 
 import datetime
 from enum import Enum
+from pathlib import Path
 from typing import *
 
 from .exceptions import *
@@ -172,7 +173,7 @@ class Loader:
     similar to each other, it is easy to obtain an unexpected type.
     """
 
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs) -> None:
         # Types that do not need conversion
         self.basictypes = {int, bool, float, str, NONETYPE}
 
@@ -213,12 +214,15 @@ class Loader:
             (is_literal, _literalload),
             (is_typeddict, _namedtupleload),
             (lambda type_: type_ in {datetime.date, datetime.time, datetime.datetime}, _datetimeload),
+            (lambda type_: type_ == Path, _pathload),
             (is_attrs, _attrload),
         ]  # type: List[Tuple[Callable[[Any], bool], Callable[[Loader, Any, Type], Any]]]
 
         for k, v in kwargs.items():
             setattr(self, k, v)
 
+        self._indexcache = {}  # type: Dict[Type, int]
+
     def index(self, type_: Type[T]) -> int:
         """
         Returns the index in the handlers list
@@ -244,14 +248,20 @@ class Loader:
         TypeError is raised if there is no known way to treat type_,
         otherwise all errors raise a ValueError.
         """
-        try:
-            index = self.index(type_)
-        except ValueError:
-            raise TypedloadTypeError(
-                'Cannot deal with value of type %s' % type_,
-                value=value,
-                type_=type_
-            )
+        p_index = self._indexcache.get(type_)
+
+        if p_index is not None:
+            index = p_index
+        else:
+            try:
+                index = self.index(type_)
+                self._indexcache[type_] = index
+            except ValueError:
+                raise TypedloadTypeError(
+                    'Cannot deal with value of type %s' % type_,
+                    value=value,
+                    type_=type_
+                )
 
         # Add type to known types, to resolve ForwardRef later on
         if self.frefs is not None and hasattr(type_, '__name__'):
@@ -362,7 +372,7 @@ def _setload(l: Loader, value, type_) -> Set:
     This loads into something like Set[int]
     """
     t = type_.__args__[0]
-    return {l.load(i, t) for i in value}
+    return {l.load(v, t, annotation=Annotation(AnnotationType.INDEX, i)) for i, v in enumerate(value)}
 
 
 def _frozensetload(l: Loader, value, type_) -> FrozenSet:
@@ -370,7 +380,7 @@ def _frozensetload(l: Loader, value, type_) -> FrozenSet:
     This loads into something like FrozenSet[int]
     """
     t = type_.__args__[0]
-    return frozenset(l.load(i, t) for i in value)
+    return frozenset(l.load(v, t, annotation=Annotation(AnnotationType.INDEX, i)) for i, v in enumerate(value))
 
 
 def _tupleload(l: Loader, value, type_) -> Tuple:
@@ -383,7 +393,7 @@ def _tupleload(l: Loader, value, type_) -> Tuple:
         args = type_.__tuple_params__
 
     if len(args) == 2 and args[1] == ...: # Tuple[something, ...]
-        return tuple(l.load(i, args[0]) for i in value)
+        return tuple(l.load(v, args[0], annotation=Annotation(AnnotationType.INDEX, i)) for i, v in enumerate(value))
     else: # Tuple[something, something, somethingelse]
         if l.failonextra and len(value) > len(args):
             raise TypedloadValueError('Value is too long for type %s' % type_, value=value, type_=type_)
@@ -475,7 +485,7 @@ def _unionload(l: Loader, value, type_) -> Any:
     try:
         args = uniontypes(type_)
     except AttributeError:
-        raise TypedloadAttributeError('The typing API for this Python version is unknown')
+        raise TypedloadAttributeError('The typing API for this Python version is unknown', type_=type_, value=value)
 
     value_type = type(value)
 
@@ -530,15 +540,17 @@ def _enumload(l: Loader, value, type_) -> Enum:
         pass
 
     # Try with the typing hints
+    exceptions = []
     for _, t in get_type_hints(type_).items():
         try:
-            return type_(l.load(value, t))
-        except:
-            pass
+            return type_(l.load(value, t, annotation=Annotation(AnnotationType.UNION, t)))
+        except Exception as e:
+            exceptions.append(e)
     raise TypedloadValueError(
         'Value could not be loaded into %s' % type_,
         value=value,
-        type_=type_
+        type_=type_,
+        exceptions=exceptions
     )
 
 
@@ -556,7 +568,7 @@ def _datetimeload(l: Loader, value, type_) -> Union[datetime.date, datetime.time
     try:
         return type_(*value)
     except TypeError as e:
-        raise TypedloadTypeError(str(e))
+        raise TypedloadTypeError(str(e), type_=type_, value=value)
 
 
 def _attrload(l, value, type_):
@@ -590,3 +602,10 @@ def _attrload(l, value, type_):
     ))
 
     return _namedtupleload(l, value, t)
+
+
+def _pathload(l: Loader, value, type_) -> Path:
+    try:
+        return Path(value)
+    except TypeError as e:
+        raise TypedloadTypeError(str(e), type_=type_, value=value)
